import os, stat
from odict import odict
import turbogears
from turbogears import controllers, expose
import cherrypy
from cherrypy.lib.cptools import serve_file

from wizard import WizardController
from status import StatusController
from help import HelpController
from config import ConfigController

from paraxip.gateway_gui_widgets.tabpstn  import TabPSTN

import pstnconfig.model as model
import logging
log = logging.getLogger("rootconfig")

class Root(controllers.RootController):

  # Determine the project path.
  # The project path is made of the PARAXIP_INSTALLDIR environment variable
  # and the project name. 
  strInstallDir = os.getenv("PARAXIP_INSTALLDIR")
  if strInstallDir is None:
    log.error("PARAXIP_INSTALLDIR environment variable not set. Cannot "\
               "determine project path")       
  # TODO: Get project name from config, Right now project name is 
  # hardcoded to pstnconfig.
  strProjectPath = os.path.join(strInstallDir, "pstnconfig")
  wizard  = WizardController(strProjectPath)
  status  = StatusController()
  help    = HelpController()
  config  = ConfigController(strProjectPath)

  #Elements that will build our tabber
  tabs = odict()
  #Each elements of the tabs dictionnary has the format:
  # tabs['Tab Text'] = ['/autoloadUrl','tabId']

  if status.isGWStarted() :
    tabs['Status and Control'] = ['/status','statusControlTabId']
  else :
    tabs['Status and Control'] = ['/static/gateway_down.html','statusControlTabId']

  tabs['Configuration'] = ['','configTabId']
  tabs['Quick Setup'] = ['','wizardTabId']
  tabs['Help'] = ['/static/help.html','helpTabId']

  items = [{'title': k, 'autoLoad': v[0], 'id': v[1]} for k, v in tabs.items()]
  tabList = [(k.replace(" ","").lower(),v[0],v[1]) for k, v in tabs.items()]
  tabs = dict(tabs)

  strFileLocation = os.path.join(strProjectPath, "pstnconfig/static/"
                                 "javascript/widgets_src/main_pstn_tabs.js")

  # Flag to switch on the offering of the virtual boards in the wizard
  # Turbogears bool configuration property 
  # "netborder.oamWebServer.offerVirtualBoards" (default: False)
  offerVirtualBoards = turbogears.config.get("netborder.oamWebServer.offerVirtualBoards", 
                                             False)

  # Check if file exists
  if model.PSTNConfigFileExists():
    # PSTN config file already exists
    log.info("PSTN config file %s already exists", model.getPSTNConfigFileName() )
    extTab = TabPSTN( tabs=tabs, items=items, renderTo="pstn_tabs", 
                      tabList=tabList, width=757, height=500, activeTab='statusControlTabId',
                      src_file=strFileLocation,
                      offerVirtualBoards=offerVirtualBoards,
                      children=[wizard.getWizardForm(),config.getSubTabs()])
  else:
    log.info("PSTN config file %s does not already exist", model.getPSTNConfigFileName() )
    extTab = TabPSTN( tabs=tabs, items=items, renderTo="pstn_tabs",
                      tabList=tabList, width=757, height=500, activeTab='wizardTabId',
                      src_file=strFileLocation,
                      offerVirtualBoards=offerVirtualBoards,
                      children=[wizard.getWizardForm(),config.getSubTabs()])

  @expose(template='pstnconfig.templates.index')
  def index(self):
    """Show the user the choice he have to edit the pstn-config.xml file."""
    # Check the product name to determine what should be written in
    # the footer of the master page
    if os.getenv("PARAXIP_PRODUCT_NAME") == "Netborder Express Gateway":
      footerText = u"\xA9 2008 Sangoma Technologies Corp."
    else:
      footerText = u"\xA9 2003-2008 Paraxip Technologies"

    return dict(extTab = self.extTab, footerText = footerText)

  @expose()
  def configFileExist(self, _dc=''):
    return {"configFileExist":model.PSTNConfigFileExists()}

  # Controller to redirect requests to the same host but different port 
  #   including the remaining path and the query parameter of the original 
  #   request (for use for ex. to redirect to the embedded gw web server on 
  #   the same host, different port)
  # path is an array of path segments (not including the segement that 
  #   triggers the call to this controller)
  # query_params is a dictionany of the query parameters
  # TODO re-useable in other projects
  def redirectToSameHostDifferentPort(self, port, *path, **query_params):

    # cherrypy.request holds the current request (different instance per thread)
    requestHeaders = cherrypy.request.headers
    # Host header is 'host [: port]' following HTTP 1.1
    assert requestHeaders.has_key("Host")
    hostAndPort = requestHeaders.get('Host')

    # Isolate host name, skip port if present
    import re
    # anything char but colon, until colon or end of string
    match = re.match('([^:]+):?', hostAndPort)
    assert match
    host = match.group(1) # first atom (what's behing parethesis in regexp)

    targeturl = '%s://%s:%d' % (cherrypy.request.scheme,
                                host,
                                port)

    # Build target URL path following remaining path segments
    for path_segment in path:
      targeturl += '/' + path_segment 
    
    import urllib
    query_string = urllib.urlencode(query_params)
    targeturl += '?' + query_string

    log.info("Target URL for redirect is '%s'" % targeturl)

    raise cherrypy.HTTPRedirect(targeturl)

  @expose()
  def redirectToEmbeddedGWGui(self, *args, **kw):
    # HARDCODED port number
    EMBEDDED_GW_GUI_PORT = 7782
    return self.redirectToSameHostDifferentPort(EMBEDDED_GW_GUI_PORT,
                                                *args,
                                                **kw)

  # Used by static html file for startup
  @expose()
  def ping(self, *args, **kw):
    #set headers to block IE caching
    cherrypy.response.headers['Expires'] = '...'
    cherrypy.response.headers['Cache-Control'] = \
      'no-store, no-cache, must-revalidate, post-check=0, pre-check=0'
    cherrypy.response.headers['Pragma'] = 'no-cache'

    if kw.has_key('delayMs'):
      # Asked to delay response for a given ms
      # Will raise ValueError if cannot be converted to int
      delayMs = int(kw['delayMs'])
      import time
      time.sleep(delayMs / 1000.0) # Using float literal to keep decimals
    
    return 'pong'

  @expose()
  def releaseNotes(self):
    filePath = os.path.join(self.strInstallDir,"doc","Release_Notes.pdf")
    log.info("release notes path: %s" % filePath)
    return serve_file(filePath)

  @expose()
  def userGuide(self):
    filePath = os.path.join(self.strInstallDir,"doc","TDM_Gateway_User_Guide.pdf")
    log.info("user guide path: %s" % filePath)
    return serve_file(filePath)

  @expose()
  def license(self):
    # TODO - put the license file name in the config file.
    if os.getenv("PARAXIP_PRODUCT_NAME") == "Netborder Express Gateway":
      filename = "netborder-express-gateway-license.txt"
    else:
      filename = "netborder-gateway-license.txt"

    filePath = os.path.join(self.strInstallDir,"config",filename)
    log.info("license file path: %s" % filePath)
    return serve_file(filePath)

  @expose()
  def default(self, *args, **kw):
    return 'Page not found'




